
//  ForwardedForFilter.java
//



import java.util.ArrayList;
import java.util.Enumeration;
import java.util.HashMap;

import com.netegrity.proxy.filter.BaseProxyFilter;
import com.netegrity.proxy.filter.ProxyFilterConfig;
import com.netegrity.proxy.filter.ProxyFilterException;
import com.netegrity.proxy.filter.ProxyRequest;
import com.netegrity.proxy.filter.ProxyResponse;

/**
 * This is a sample ProxyFilter demonstrating how filters may be used as
 * pre filters.
 * 
 * Note: This filter is activated through the proxy rules.  See
 * proxy rules documentation for more.
 */
public class ForwardedForFilter extends BaseProxyFilter {

	int debugTraceLevel = 0;

	static final int DEBUGLEVEL_None = 0;
	static final int DEBUGLEVEL_Basic = 1;
	static final int DEBUGLEVEL_Trace = 2;
	
	boolean isBasicDebug() { return debugTraceLevel > DEBUGLEVEL_None; }
	boolean isTraceDebug() { return debugTraceLevel > DEBUGLEVEL_Basic; }  
	
    String forwardForHeaderName = "";
    boolean overwrite = false;
    boolean enabled = true; 

    static boolean isTrue(String value, boolean defaultValue) {

        if (value == null) return defaultValue;

        if (value.startsWith("t")) {
             return true;
        }

        if (value.startsWith("T")) {
             return true;
        }

        if (value.startsWith("f")) {
             return true;
        }

        if (value.startsWith("F")) {
             return true;
        }

        return defaultValue;
        
    }

    /*
     * In init() we read parameters from the server.conf config file.  
     *     X_FORWARD_FOR_HDR_NAME = X-Forward-For
     *     MODE = (APPEND|OVERWRITE)  default to APPEND
     *     ENABLED = (True|False)  default to False
     */	
	public void init(ProxyFilterConfig config) throws ProxyFilterException {
		super.init(config);
		
		// init debug trace
		String param = config.getInitParameter("debugTraceLevel");

		if (isTraceDebug()) { // wont work unless debug level set in code 		
			System.out.println("Param debugTraceLevel : x" + param + "x");
		}
		if (param != null ) {
			try { 
				debugTraceLevel = Integer.parseInt(param);
			}
			catch(NumberFormatException e) {} // Ignore will be 0.
		
		}

        forwardForHeaderName = config.getInitParameter("XForwardForHeaderName");
		
        if (forwardForHeaderName == null) {
            forwardForHeaderName = "X-Forwarded-For";
        }

        String mode = config.getInitParameter("mode");  // will be OVERWRITE or APPEND (default to APPEND) 

        if (mode != null && mode.equalsIgnoreCase("OVERWRITE")) {
           overwrite = true;
        }

        enabled = isTrue(config.getInitParameter("enabled"), true);  // Check if enabled 

	    if (isBasicDebug()) {
            mode = "Append"; 
            if (overwrite) mode = "Overwrite";

			System.out.println("ForwardedForFilter.init : EXECUTING FILTER INIT  " +  config.getFilterName());
			System.out.println("XForwardForHeaderName   " +  forwardForHeaderName);
			System.out.println("MODE                    " +  mode);
			System.out.println("ENABLED                 " +  enabled);
            

        }

		// if we really need an exceiton.
		//throw new ProxyFilterException("ForwardedForFilter : missing required init parameter");
	}

	/**
	 * Prefilter process to update the X-Forwarded-For: header. 
	 */
	public void doPreFilter(ProxyRequest prequest, ProxyResponse presponse) throws ProxyFilterException {

		if (isTraceDebug()) {
			System.out.println("ForwardedForFilter.doPreFilter : Start filtering request "+  getFilterConfig().getFilterName());
		}

        String remoteAddr = prequest.getOriginalRequest().getRemoteAddr();

		String forwardValue = prequest.getHeader(forwardForHeaderName);

	    if (isBasicDebug()) {
		    System.out.println("ForwardedForFilter.doPreFilter :  orig value: " +  forwardForHeaderName + ": " + forwardValue);
		    System.out.println("ForwardedForFilter.doPreFilter :  remoteAddr: " +  remoteAddr);
	    }

        if (remoteAddr == null || remoteAddr.length() == 0 ) { // Check we have value value = 
		    System.out.println("ForwardedForFilter.doPreFilter :  remoteAddr value is null or empty ");
        }

        else if (enabled) {  
		    if (forwardValue == null || forwardValue.length() == 0 ) {
                prequest.setHeader(forwardForHeaderName, remoteAddr);
            }
            else if (overwrite) {
                // TODO - Check if set will do an update - rather than remove and add 
			    prequest.removeHeader(forwardForHeaderName);
                prequest.setHeader(forwardForHeaderName, remoteAddr);
            }
            else { // append 
                forwardValue = forwardValue + ", " + remoteAddr;

                // TODO - Check if set will do an update - rather than remove and add 
			    prequest.removeHeader(forwardForHeaderName);
                prequest.setHeader(forwardForHeaderName, forwardValue);
            }
             
        }

	    if (isBasicDebug()) {
    		forwardValue = prequest.getHeader(forwardForHeaderName);
		    System.out.println("ForwardedForFilter.doPreFilter :  new  value: " +  forwardForHeaderName + ": " + forwardValue);
	    }

		if (isTraceDebug()) {
			System.out.println("SamplePreFilter.doPreFilter : Finished filtering request "+  getFilterConfig().getFilterName());
		}
	}
	
	public void doPostFilter(ProxyRequest prequest, ProxyResponse presponse)  throws ProxyFilterException {
		// do nothing here.
	}
}



